#pragma implementation

#include <debtags/BasicPackageMatcher.h>
#include <debtags/Package.h>
#include <debtags/Maintainer.h>

#include <Regexp.h>

using namespace std;
using namespace Tagcoll;
using namespace Debtags;

BasicPackageMatcher::~BasicPackageMatcher() throw ()
{
	if (name) delete name;
	if (str) delete str;
}

void BasicPackageMatcher::setName(const std::string _name) throw ()
{
	try {
		if (_name.empty())
		{
			if (name) delete name;
			name = 0;
		} else
			name = new Regexp(_name);
	} catch (RegexpException& e) {
		fprintf(stderr, "%s: %.*s\n", e.type(), PFSTR(e.desc()));
		name = 0;
	}
}

void BasicPackageMatcher::setString(const std::string _str) throw ()
{
	try {
		if (_str.empty())
		{
			if (str) delete str;
			str = 0;
		} else
			str = new Regexp(_str);
	} catch (RegexpException& e) {
		fprintf(stderr, "%s: %.*s\n", e.type(), PFSTR(e.desc()));
		str = 0;
	}
}

bool BasicPackageMatcher::matchName(const std::string& text) const throw ()
{
	if (name == 0)
		return true;
	return name->match(text);
}
bool BasicPackageMatcher::matchMaintainer(const std::string& text) const throw ()
{
	if (maint.empty())
		return true;
	return text.find(maint) != string::npos;
}
bool BasicPackageMatcher::matchString(const std::string& text) const throw ()
{
	if (str == 0)
		return true;
	return str->match(text);
}
bool BasicPackageMatcher::matchInstalled(int status) const throw ()
{
	if (installed == ANY)
		return true;
	if (installed == YES && status != 0 && status != 5)
		return true;
	if (installed == NO && (status == 0 || status == 5))
		return true;
	return false;
}
bool BasicPackageMatcher::matchDebtags(const TagSet& ptags) const throw ()
{
	if (tags.empty())
		return true;
	return ptags.contains(tags);
}

bool BasicPackageMatcher::match(const Package& pkg) const throw ()
{
	bool got = true;
	if (hasMaintainer() && pkg.maint() && !matchMaintainer(pkg.maint().fullname()))
		got = false;
	if (hasName() && !matchName(pkg.name()))
		got = false;
	if (hasString() && !matchString(pkg.name()) && !matchString(pkg.sdesc()) && !matchString(pkg.ldesc()))
		got = false;
	try {
		if (hasInstalled() && !matchInstalled(pkg.state()))
			got = false;
	} catch (PackageDBException& e) {
		fprintf(stderr, "%s: %.*s. Skipping check for package status\n", e.type(), PFSTR(e.desc()));
	}
	if (hasDebtags() && !matchDebtags(pkg.tags()))
		got = false;
	return got;
}

// vim:set ts=4 sw=4:
