#include <apt-pkg/tagfile.h>
#include <apt-pkg/error.h>

#include <tagcoll/stringf.h>
#include <tagcoll/Exception.h>

#include <debtags/Environment.h>
#include <debtags/Package.h>
#include <debtags/Maintainer.h>

#include <debtags/PackageDB.h>


using namespace std;
using namespace Debtags;

#define TEST_PKG "test"
#define TEST_MAINT "Enrico Zini <enrico@debian.org>"

class PkgRecordPrinter : public PkgRecordConsumer
{
public:
	virtual void operator()(pkgRecords::Parser& p)
	{
		string name = p.Name();
		string desc = p.ShortDesc();

		fprintf(stderr, "%.*s: %.*s\n", PFSTR(name), PFSTR(desc));
	}
};

int result = 0;
int errormaints = 0;

class PkgRecordChecker : public PkgRecordConsumer
{
public:
	virtual void operator()(pkgRecords::Parser& p)
	{
		string name = p.Name();
		string sdesc = p.ShortDesc();
		string ldesc = p.LongDesc();
		string maint = p.Maintainer();

		Package pkg = Debtags::Environment::get().getPackage(name);

		// Compare data in section and pkg
		if (!pkg)
		{
			result = 1;
			fprintf(stderr, "%.*s: package not found\n", PFSTR(name));
		}
		else if (pkg.name() != name)
		{
			result = 1;
			fprintf(stderr, "%.*s: package name \"%.*s\" is different than expected\n",
					PFSTR(name), PFSTR(pkg.name()));
		}
		else if (!pkg.maint())
		{
			result = 1;
			fprintf(stderr, "%.*s: null maintainer from pkg.maint()\n", PFSTR(name));
		}
		else if (pkg.maint().email() == "error")
		{
			errormaints++;
		}
		else if (pkg.maint().fullname() != maint)
		{
			result = 1;
			string t = pkg.maint().fullname();
			fprintf(stderr, "%.*s: Package maintainer \"%.*s\" is different than expected \"%.*s\"\n",
					PFSTR(name), PFSTR(t), PFSTR(maint));
		}
		//else
			//printf("%.*s %.*s\n", PFSTR(pkg.name()), PFSTR(pkg.sdesc()));
	}
};


int main (int argc, char *argv[])
{
	try {
		// Install the handler for unexpected exceptions
		InstallUnexpected installUnexpected;

		Debtags::Environment::init(false);

		/*
		PackageDB db;
		PkgRecordPrinter printer;
		db.iterateAll(printer);
		return result;
		*/


		PackageDB db;
		PkgRecordChecker checker;
		db.iterateAll(checker);

		if (errormaints)
			fprintf(stderr, "Found %d invalid packages\n", errormaints);



		/*

		string test = "Package: " TEST_PKG "\n"
			          "Maintainer: " TEST_MAINT "\n"
					  "Description: This is a test non-package\n"
					  " In reality, this is just a test package record\n"
					  " designed to test a personal way to use the libapt-pkg\n"
					  " package record scanner\n\n";

		// Parse the package data
		pkgTagSection section;
		section.Scan(test.data(), test.size());

		string t;
		t = section.FindS("Package");
		if (t != TEST_PKG)
		{
			result = 1;
			fprintf(stderr, "Package is \"%.*s\" instead of \"" TEST_PKG "\"\n", PFSTR(t));
		}

		t = section.FindS("Maintainer");
		if (t != TEST_MAINT)
		{
			result = 1;
			fprintf(stderr, "Maintainer is \"%.*s\" instead of \"" TEST_MAINT "\"\n", PFSTR(t));
		}

		t = section.FindS("Description");
		//fprintf(stderr, "Description is \"%.*s\"\n", PFSTR(t));

		char buf[256];
		while (fgets(buf, 255, stdin))
		{
			string name = buf;
			name.resize(name.size() - 1);
			Package pkg = Debtags::Environment::get().getPackage(name);
			if (!pkg)
			{
				result = 1;
				fprintf(stderr, "Package not found: \"%.*s\"\n", PFSTR(name));
			}
			else if (pkg.name() != name)
			{
				result = 1;
				fprintf(stderr, "Package name \"%.*s\" is different than expected \"%.*s\"\n",
						PFSTR(pkg.name()), PFSTR(name));
			}
			else
				printf("%.*s %.*s\n", PFSTR(pkg.name()), PFSTR(pkg.desc()));
		}
		*/
		return result;
	} catch (Exception& e) {
		fprintf(stderr, "%s: %.*s\n", e.type(), PFSTR(e.desc()));
	}
}

// vim:set ts=4 sw=4:
