#include <debtags/Environment.h>
#include <debtags/PackageSet.h>
#include <debtags/TagDB.h>

#include <tagcoll/TagSet.h>
#include <tagcoll/InputMerger.h>

#include <set>

using namespace std;
using namespace Debtags;

class PackageGatherer : public TagcollConsumer<Package, Tag>
{
protected:
	PackageSet coll;
public:
	virtual ~PackageGatherer() throw () {}
	virtual void consume(const Package& item) throw ()
	{
		coll += item;
	}
	virtual void consume(const Package& item, const OpSet<Tag>& tags) throw ()
	{
		coll += item;
	}
	PackageSet& getColl() throw () { return coll; }
};


int main (int argc, char *argv[])
{
	try {
		// Install the handler for unexpected exceptions
		InstallUnexpected installUnexpected;

		Debtags::Environment::init(false);

		int result = 0;
		
		// Test set operations on items of type Package
		PackageSet test;
		test.insert(Debtags::Environment::get().getPackage("apt"));
		test.insert(Debtags::Environment::get().getPackage("apt"));
		test.insert(Debtags::Environment::get().getPackage("debtags"));
		Package p = Debtags::Environment::get().getPackage("debtags");
		test.insert(p);
		if (test.size() != 2)
		{
			fprintf(stderr, "%d items (I want 2)\n", test.size());
			result = 1;
		}
		test.erase(p);
		if (test.size() != 1)
		{
			fprintf(stderr, "%d items (I want 1)\n", test.size());
			result = 1;
		}

		// Compare two methods of getting tags
		TagDB first = Debtags::Environment::get().tagDB();
		InputMerger<string, string> second;
		TagDB::outputPatched(second);
		PackageGatherer pg;
		TagDB::outputPatched(pg);
		for (PackageSet::const_iterator i = pg.getColl().begin(); i != pg.getColl().end(); i++)
		{
			TagSet tsfirst = i->tags();
			OpSet<string> tssecond = second.getTagsetForItem(i->name());
			
			TagSet::const_iterator fi = tsfirst.begin();
			OpSet<string>::const_iterator si = tssecond.begin();
			for ( ; fi != tsfirst.end() && si != tssecond.end(); fi++, si++)
			{
				if (fi->fullname() != *si)
				{
					fprintf(stderr, "%.*s: tag differ [debtags]%.*s != [tagcoll]%.*s\n",
							PFSTR(i->name()), PFSTR(fi->fullname()), PFSTR(*si));
					result = 1;
				}
			}
			for (; fi != tsfirst.end(); fi++)
			{
				fprintf(stderr, "%.*s: tag only in debtags: %.*s\n",
						PFSTR(i->name()), PFSTR(fi->fullname()));
				result = 1;
			}
			for (; si != tssecond.end(); si++)
			{
				fprintf(stderr, "%.*s: tag only in tagcoll: %.*s\n",
						PFSTR(i->name()), PFSTR(*si));
				result = 1;
			}
		}

		return result;
	} catch (Exception& e) {
		fprintf(stderr, "%s: %.*s\n", e.type(), PFSTR(e.desc()));
	}
}

// vim:set ts=4 sw=4:
